from abc import abstractmethod

from omnibridge.model_entities.models_io.base_model_io import ModelIO
from omnibridge.saved_data.json_data_manager import JsonConvertable


class ModelWrapper(JsonConvertable):
    @abstractmethod
    def process(self, model_input: ModelIO) -> ModelIO:
        pass

    @abstractmethod
    def get_name(self) -> str:
        pass

    @classmethod
    @abstractmethod
    def get_class_type_field(cls) -> str:
        pass



from abc import ABC, abstractmethod
from typing import Any, Type

from omnibridge.model_entities.models_io.base_model_io import ModelIO


class BaseModelUnit(ABC):
    def can_process(self, model_input: ModelIO) -> bool:
        return self.can_process_type(type(model_input))

    @abstractmethod
    def can_process_type(self, model_input_type: Type[ModelIO]) -> bool:
        pass

    @abstractmethod
    def produced_type(self) -> Any:
        pass

    @abstractmethod
    def process(self, model_input: ModelIO) -> ModelIO:
        pass




from typing import Any, Dict, Union
import json
import logging

from .rest_api_wrapper import RestAPIWrapper
from ...model_entities.models_io.base_model_io import ModelIO, TextualIO

HUGGING_FACE_BASE_URL = "https://api-inference.huggingface.co/models"

SUPPORTED_TASKS = [
    "text2text-generation",
    "text-generation"
]

class HuggingFaceWrapper(RestAPIWrapper):
    def __init__(self, name: str, api_key: str, model: str, logger: logging.Logger = logging.getLogger()) -> None:
        """
        :param: model - hugging face repo
        """
        super().__init__(name, logger)
        self.api_url = HUGGING_FACE_BASE_URL
        self.api_key = api_key
        self.model = model

        from huggingface_hub import HfApi
        api = HfApi()
        model = api.model_info(repo_id=model)

        if model is None:
            raise Exception('Repo id do not exists')
        
        self.task = None
        for task in SUPPORTED_TASKS:
            if task in model.tags:
                self.task = task
                break
        
        if self.task is None:
            raise Exception('Task is not supported')            

    def _get_api_key(self) -> str:
        return self.api_key

    def process(self, model_input: ModelIO) -> ModelIO:
        if not isinstance(model_input, TextualIO):
            raise NotImplementedError
        
        response = self.prompt(prompt_message=model_input.text)

        if "error" in response:
            raise ValueError(
                f"Error raised by inference API: {response['error']}"
            )

        if self.task == "text-generation":
            # Text response includes the prompt text.
            data = response[0]["generated_text"][len(model_input.text) :]
        elif self.task == "text2text-generation":
            data = response[0]["generated_text"]
        else:
            raise Exception(f'Task {self.task} is not supported')

        return TextualIO(data)
        

    def to_json(self) -> Dict[str, Union[str, int]]:
        return {
            "api key": self.api_key,
            "model": self.model,
            "_class_type": self.get_class_type_field()
        }

    @classmethod
    def create_from_json(cls, json_key: str, json_data: Dict[str, str]) -> Any:
        return HuggingFaceWrapper(json_key, api_key=json_data["api key"], model=json_data["model"])

    @classmethod
    def get_description(cls) -> str:
        return """
            Huggin face wrapper, allow accessing any model in hugging face model that supports API endpoint
        """

    @classmethod
    def get_class_type_field(cls) -> str:
        return "hugging_face"

    def _get_body(self, prompt_message: str) -> Any:
        return json.dumps({
            "inputs": prompt_message
        })

    def _get_api_url(self) -> str:
        return f"{self.api_url}/{self.model}"



import os
from typing import Union

from omnibridge.model_entities.models_io.base_model_io import TextualIO


class FileInputHandler:
    @staticmethod
    def convert_file(file_path: str) -> Union[TextualIO, None]:
        """
        Reads the content of a file given its file path.

        Args:
            file_path (str): The path to the file.

        Returns:
            TextualIO: The content of the file ready to be consumed by models.
        """

        allowed_extensions = {
            '.txt': FileInputHandler.convert_txt_file,
            '.py': FileInputHandler.convert_txt_file
        }

        _, file_extension = os.path.splitext(file_path)

        if file_extension.lower() not in allowed_extensions.keys():
            raise ValueError(f"Error: The provided file must be of the following types: {allowed_extensions.keys()}.")

        file_conversion_func = allowed_extensions[file_extension]

        return file_conversion_func(file_path)

    @staticmethod
    def convert_txt_file(file_path: str) -> TextualIO:
        """
        Reads the content of a .txt file given its file path.

        Args:
            file_path (str): The path to the file.

        Returns:
            TextualIO: The content of the file ready to be consumed by models.
        """

        with open(file_path, 'r', encoding='utf-8') as file:
            content = file.read()
        return TextualIO(content)



from typing import Dict, Union, Any

from omnibridge.saved_data.json_data_manager import JsonConvertable


class ApiKey(JsonConvertable):
    def __init__(self, value: str):
        self.value = value

    def to_json(self) -> Dict[str, Union[str, int]]:
        return {
            'value': self.value
        }

    @classmethod
    def create_from_json(cls, json_key: str, json_data: Dict[str, str]) -> Any:
        return ApiKey(json_data['value'])



from setuptools import setup, find_packages
from importlib import util
from os import path
import os

this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, "README.md"), encoding="utf-8") as f:
    long_description = f.read()

spec = util.spec_from_file_location(
    "omnibridge.version", os.path.join("omnibridge", "version.py")
)
mod = util.module_from_spec(spec)  # type: ignore[arg-type]
spec.loader.exec_module(mod)  # type: ignore[union-attr]
version = mod.version


setup(
    name='OmniBridge',
    license="Apache License 2.0",
    version=version,
    author='OmniSpective',
    author_email='eliran9692@gmail.com',
    description='Bridging AI models',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url='https://github.com/OmniSpective/OmniBridge',
    packages=find_packages(exclude=["cli*", "version*", "main*"]),
    install_requires=[
        'requests',
        'ruff',
        'argparse'
    ],
    entry_points={
            "console_scripts": [
                "omnibridge=omnibridge.main:main",
                "obr=omnibridge.main:main",
            ]
        },
)



import argparse
from omnibridge.cli.banner import banner
from omnibridge.cli.create.create_parser import add_create_arguments
from omnibridge.cli.run.run_parser import add_run_arguments
from omnibridge.cli.list.list_parser import add_list_arguments
from omnibridge.cli.create.create_command_handler import handle_create_command
from omnibridge.cli.run.run_command_handler import handle_run_command
from omnibridge.cli.list.list_command_handler import handle_list_command


COMMAND_TO_HANDLER = {
    'create': handle_create_command,
    'list': handle_list_command,
    'run': handle_run_command
}


def run() -> int:
    parser = argparse.ArgumentParser(description='AI integration tool.')
    subparsers = parser.add_subparsers(help='sub-command help', dest='command')

    create_parser = subparsers.add_parser("create", help="Create models/keys/flows.")
    create_subparsers = create_parser.add_subparsers(help='create sub-command help', dest='object_to_create')

    list_parser = subparsers.add_parser("list", help="List keys/models/flows/wrappers")
    list_subparsers = list_parser.add_subparsers(help="list sub-command help", dest="object_to_list")

    run_parser = subparsers.add_parser("run", help="Run model/flow")
    run_subparsers = run_parser.add_subparsers(help="run sub-command help", dest="object_to_run")

    add_create_arguments(create_subparsers)
    add_list_arguments(list_subparsers)
    add_run_arguments(run_subparsers)
    
    args = vars(parser.parse_args())

    handler = COMMAND_TO_HANDLER[args['command']]
    handler(args)

    return 0


def main():
    print(banner)
    exit(run())


if __name__ == '__main__':
    main()
